import QtQuick 2.0
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import QtQuick.Controls.Styles 1.0
import im.ricochet 1.0

ToolBar {
    Layout.minimumWidth: 200
    Layout.fillWidth: true
    // Necessary to avoid oversized toolbars, e.g. OS X with Qt 5.4.1
    implicitHeight: toolBarLayout.height + __style.padding.top + __style.padding.bottom

    property Action addContact: addContactAction
    property Action preferences: preferencesAction

    data: [
        Action {
            id: addContactAction
            enabled: userIdentity.hostOnionServiceState != UserIdentity.HostOnionServiceState_None
            //: Tooltip text for the button that launches the dialog box for adding a new contact
            text: qsTr("Add Contact")
            onTriggered: {
                var object = createDialog("AddContactDialog.qml", { }, window)
                object.visible = true
            }
        },

        Action {
            id: preferencesAction
            //: Tooltip text for the button that launches program preferences window
            text: qsTr("Preferences")
            onTriggered: root.openPreferences()
        }
    ]

    SystemPalette {
        id: paletteActive
        colorGroup: SystemPalette.Active
    }

    SystemPalette {
        id: paletteDisabled
        colorGroup: SystemPalette.Disabled
    }

    Component {
        id: iconButtonStyle

        ButtonStyle {
            background: Item { }
            label: Text {
                text: control.text
                font.family: iconFont.name
                font.pixelSize: height
                horizontalAlignment: Qt.AlignHCenter
                renderType: Text.QtRendering
                color: paletteActive.text
            }
        }
    }

    Component {
        id: iconDisabledButtonStyle

        ButtonStyle {
            background: Item { }
            label: Text {
                text: control.text
                font.family: iconFont.name
                font.pixelSize: height
                horizontalAlignment: Qt.AlignHCenter
                renderType: Text.QtRendering
                color: paletteDisabled.text
            }
        }
    }

    RowLayout {
        id: toolBarLayout
        width: parent.width

        TorStateWidget {
            Layout.alignment: Qt.AlignVCenter
        }

        Item {
            Layout.fillWidth: true
            height: 1
        }

        ToolButton {
            id: addContactButton
            implicitHeight: 24
            action: addContactAction
            style:  userIdentity.hostOnionServiceState == UserIdentity.HostOnionServiceState_None ? iconDisabledButtonStyle : iconButtonStyle
            text: "\ue810" // iconFont plus symbol

            Loader {
                id: emptyState
                active: contactList.view.count == 0
                sourceComponent: Bubble {
                    target: addContactButton
                    maximumWidth: toolBarLayout.width
                    //: Tooltip that displays on first launch indicating how to add a new contact
                    text: qsTr("Click to add contacts")
                }
            }

            Accessible.role: Accessible.Button
            //: Name of the button for adding a new contact for accessibility tech like screen readers
            Accessible.name: qsTr("Add Contact")
            //: Description of the 'Add Contact' button for accessibility tech like screen readers
            Accessible.description: qsTr("Shows the add contact dialogue")
        }

        ToolButton {
            action: preferencesAction
            implicitHeight: 24
            style: iconButtonStyle
            text: "\ue803" // iconFont gear

            Accessible.role: Accessible.Button
            //: Name of the button for launching the preferences window for accessibility tech like screen readres
            Accessible.name: qsTr("Preferences")
            //: Description of the 'Preferences' button for accessibility tech like screen readers
            Accessible.description: qsTr("Shows the preferences dialogue") // todo: translation
        }
    }
}
